#!/usr/bin/env python
# -*- coding: utf-8 -*-

import datetime
import googleapiclient
from googleapiclient import discovery
from pathlib import Path
from httplib2 import Http
import oauth2client
from oauth2client import file, client, tools


def build_json(date, note, start_time, end_time, location):

    print('Building JSON event object...')

    '''
    This works in Python 3.x but throws an exception in Python 2.7:
    TypeError: Required argument 'tz' (pos 1) not found
    '''
    utc_offset = str(datetime.datetime.now().astimezone().isoformat())[-6:]

    event = {
        'summary': '',
        'location': '',
        'start': {
            'dateTime': ''
        },
        'end': {
            'dateTime': ''
        },
        'reminders': {
            'useDefault': False,
            'overrides': [
                {'method': 'popup', 'minutes': 30},
            ],
        },
    }

    event['summary'] = note

    if location:
        event['location'] = location
    else:
        del event['location']

    '''
    Required format for dateTime values:
    2018-12-27T13:45:00-07:00
    '''

    if date and start_time:
        if end_time:
            event['start']['dateTime'] = date + 'T' + start_time + ':00' + utc_offset
            event['end']['dateTime'] = date + 'T' + end_time + ':00' + utc_offset
        else:
            event['start']['dateTime'] = date + 'T' + start_time + ':00' + utc_offset
            end_time = datetime.datetime.strptime(start_time, '%H:%M') + datetime.timedelta(minutes=30)
            end_time = datetime.datetime.strftime(end_time, '%H:%M')
            event['end']['dateTime'] = date + 'T' + end_time + ':00' + utc_offset
    elif date and end_time:
        event['start']['dateTime'] = date + 'T00:00:00' + utc_offset
        event['end']['dateTime'] = date + 'T' + end_time + ':00' + utc_offset
    else:
        event['start']['date'] = date
        del event['start']['dateTime']
        event['end']['date'] = date
        del event['end']['dateTime']

    return event


def call_api(json_arg, credentials_path, calendar_id):

    print('Calling Google Calendar API...')
    try:
        flags = tools.argparser.parse_args(args=[])
    except ImportError:
        flags = None

    '''
    If you want to change the default credentials directory
    just change path = '$home/.py2gc/' to the correct path.
    '''
    if credentials_path:
        path = credentials_path
    else:
        path = str(Path.home()) + '/.py2gc/'

    if calendar_id:
        calendar_id = str(calendar_id)
    else:
        calendar_id = 'primary'

    SCOPES = 'https://www.googleapis.com/auth/calendar'
    store = file.Storage('%sstorage.json' % path)
    creds = store.get()
    if not creds or creds.invalid:
        try:
            flow = client.flow_from_clientsecrets('%sclient_id.json' % path, SCOPES)
        except (FileNotFoundError, oauth2client.clientsecrets.InvalidClientSecretsError):
            print('Cannot find client_id.json or storage.json to obtain credentials.')
            return
        creds = tools.run_flow(flow, store, flags) if flags else tools.run(flow, store)
    google_calendar = discovery.build('calendar', 'v3', http=creds.authorize(Http()))

    try:
        api_event = google_calendar.events().insert(calendarId=calendar_id,
                                                    sendNotifications=True, body=json_arg).execute()
    except googleapiclient.errors.HttpError:
        print('Cannot find calendar. Wrong ID?')
        return

    if api_event:
        event_summary = api_event['summary']
        try:
            event_date_datetime = datetime.datetime.strptime(api_event['start']['dateTime'][:10], '%Y-%M-%d')
        except KeyError:
            event_date_datetime = datetime.datetime.strptime(api_event['start']['date'], '%Y-%M-%d')
        event_date = datetime.datetime.strftime(event_date_datetime, '%d-%M-%Y')
        print('Success! Event created on %s: %s' % (event_date, event_summary))
    else:
        print('API call failed for some reason.')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import datetime


def parse_date(date):
    '''
    Take an input in the format DDMM(YYYY), check whether it is a valid date,
    add the current or next calendar year if no YYYY is given, and return
    a string in the format "YYYY-MM-DD".
    '''
    print('Parsing date...')

    current_year = datetime.datetime.now().year

    if len(date) < 4 or len(date) > 8 or not date.isdigit():
        print('Cannot parse date: %s\nDate must be in format DDMM or DDMMYYYY.' % date)
    elif len(date) == 4:
        try:
            date_to_datetime_object = datetime.datetime.strptime(date, '%d%m')
            datestr_current_year = str(current_year) + '-' + datetime.datetime.strftime(date_to_datetime_object, '%m-%d')
            datetime_current_year = datetime.datetime.strptime(datestr_current_year, '%Y-%m-%d')
            if datetime_current_year - datetime.datetime.now() > datetime.timedelta(days=0):
                parsed_date = datestr_current_year
                return parsed_date
            else:
                parsed_date = str(current_year + 1) + datestr_current_year[4:]
                return parsed_date
        except ValueError:
            print('Cannot parse date: %s\nPlease enter a valid date.' % date)
    else:
        try:
            date_to_datetime_object = datetime.datetime.strptime(date, '%d%m%Y')
            parsed_date = datetime.datetime.strftime(date_to_datetime_object, '%Y-%m-%d')
            return parsed_date
        except ValueError:
            print('Cannot parse date: %s\nPlease enter a valid date.' % date)


def parse_location(location):

    print('Parsing location...')
    parsed_location = str(location)
    return parsed_location


def parse_note(note):

    print('Parsing note...')
    parsed_note = str(note)
    return parsed_note


def parse_time(event_time):
    '''
    Take an input in the form HHMM, check whether it is a valid time,
    and return as a string in the format "HH:MM".
    '''

    print('Parsing time...')
    if len(event_time) < 4 or len(event_time) > 4 or not event_time.isdigit():
        print('Cannot parse time: %s\nTime must be in format HHMM.' % event_time)
    else:
        try:
            time_to_datetime_object = datetime.datetime.strptime(event_time, '%H%M')
            parsed_time = datetime.datetime.strftime(time_to_datetime_object, '%H:%M')
            return parsed_time
        except ValueError:
            print('Cannot parse time: %s\nPlease enter a valid time.' % event_time)


def parse_path(path):

    print('Parsing path...')
    parsed_path = str(path)
    if os.path.isdir(path):
        return parsed_path
    else:
        print('''Cannot parse path to credentials. Will try to get credentials
              from default path $home/.py2gc/.''')
        return None



#!/usr/bin/env python

from setuptools import setup

with open("README.md", 'r') as f:
    long_description = f.read()

setup(
    name='py2gc',
    version='1.0.5',
    description='Add events to a Google Calendar from the command line.',
    license="MIT",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Stefan Pfeuffer',
    author_email='mail@stefanpfeuffer.com',
    url="https://github.com/stefanpf/py2gc",
    packages=['py2gc'],
    scripts=['bin/py2gc', 'bin/py2gc.cmd'],
    install_requires=['httplib2', 'argparse', 'oauth2client', 'google-api-python-client', 'pathlib'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
